package kr.co.vividnext.sodalive.audio_content.comment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentCommentReplyViewModel(
    private val repository: AudioContentCommentRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _commentList = MutableLiveData<List<GetAudioContentCommentListItem>>()
    val commentList: LiveData<List<GetAudioContentCommentListItem>>
        get() = _commentList

    var page = 1
    private var isLast = false
    private val size = 10

    fun getCommentReplyList(commentId: Long, onFailure: (() -> Unit)? = null) {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getAudioContentCommentReplyList(
                    commentId = commentId,
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                page += 1
                                if (it.data.items.isNotEmpty()) {
                                    _commentList.postValue(it.data.items)
                                } else {
                                    isLast = true
                                    _commentList.postValue(listOf())
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }

                                if (onFailure != null) {
                                    onFailure()
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                            if (onFailure != null) {
                                onFailure()
                            }
                        }
                    )
            )
        }
    }

    fun registerComment(contentId: Long, commentId: Long, comment: String) {
        if (!_isLoading.value!!) {
            _isLoading.value = true
        }

        compositeDisposable.add(
            repository.registerComment(
                contentId = contentId,
                comment = comment,
                parentId = commentId,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false

                        if (it.success) {
                            page = 1
                            isLast = false
                            getCommentReplyList(commentId)
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun modifyComment(
        commentId: Long,
        parentCommentId: Long,
        comment: String? = null,
        isActive: Boolean? = null
    ) {
        if (comment == null && isActive == null) {
            _toastLiveData.postValue("변경사항이 없습니다.")
            return
        }

        if (comment != null && comment.isBlank()) {
            _toastLiveData.postValue("내용을 입력하세요")
            return
        }

        _isLoading.value = true

        val request = ModifyCommentRequest(commentId = commentId)

        if (comment != null) {
            request.comment = comment
        }

        if (isActive != null) {
            request.isActive = isActive
        }

        compositeDisposable.add(
            repository.modifyComment(
                request = request,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false

                        if (it.success) {
                            page = 1
                            isLast = false
                            getCommentReplyList(parentCommentId)
                        } else {
                            val message = it.message ?: "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            _toastLiveData.postValue(message)
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
