package kr.co.vividnext.sodalive.audio_content.main.v2.home

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.AudioContentRepository
import kr.co.vividnext.sodalive.audio_content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.audio_content.series.GetSeriesListResponse
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.explorer.GetExplorerSectionResponse
import kr.co.vividnext.sodalive.settings.event.EventItem
import kr.co.vividnext.sodalive.settings.notice.NoticeItem

class AudioContentMainTabHomeViewModel(
    private val repository: AudioContentMainTabHomeRepository,
    private val contentRepository: AudioContentRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _noticeLiveData = MutableLiveData<NoticeItem>()
    val noticeLiveData: LiveData<NoticeItem>
        get() = _noticeLiveData

    private var _contentBannerLiveData = MutableLiveData<List<GetAudioContentBannerResponse>>()
    val contentBannerLiveData: LiveData<List<GetAudioContentBannerResponse>>
        get() = _contentBannerLiveData

    private val _rankCreatorLiveData = MutableLiveData<GetExplorerSectionResponse>()
    val rankCreatorLiveData: LiveData<GetExplorerSectionResponse>
        get() = _rankCreatorLiveData

    private var _rankSeriesLiveData = MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val rankSeriesLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _rankSeriesLiveData

    private var _rankContentSortListLiveData = MutableLiveData<List<String>>()
    val rankContentSortListLiveData: LiveData<List<String>>
        get() = _rankContentSortListLiveData

    private var _rankContentLiveData = MutableLiveData<List<GetAudioContentRankingItem>>()
    val rankContentLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _rankContentLiveData

    private val _eventLiveData = MutableLiveData<List<EventItem>>()
    val eventLiveData: LiveData<List<EventItem>>
        get() = _eventLiveData

    private val _contentRankCreatorListLiveData = MutableLiveData<List<ContentCreatorResponse>>()
    val contentRankCreatorListLiveData: LiveData<List<ContentCreatorResponse>>
        get() = _contentRankCreatorListLiveData

    private val _salesRankContentListLiveData = MutableLiveData<List<GetAudioContentRankingItem>>()
    val salesRankContentListLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _salesRankContentListLiveData

    private val _salesCountRankContentListLiveData =
        MutableLiveData<List<GetAudioContentRankingItem>>()
    val salesCountRankContentListLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _salesCountRankContentListLiveData

    fun fetchData() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentMainHome(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            if (data.latestNotice != null) {
                                _noticeLiveData.value = data.latestNotice!!
                            }
                            _contentBannerLiveData.value = data.bannerList
                            _rankCreatorLiveData.value = data.rankCreatorList
                            _rankSeriesLiveData.value = data.rankSeriesList
                            _rankContentLiveData.value = data.rankContentList
                            _rankContentSortListLiveData.value = data.rankSortTypeList
                            _eventLiveData.value = data.eventBannerList.eventList
                            _contentRankCreatorListLiveData.value = data.contentRankCreatorList
                            _salesRankContentListLiveData.value = data.salesRankContentList
                            _salesCountRankContentListLiveData.value =
                                data.salesCountRankContentList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun getContentRanking(sort: String = "매출") {
        _isLoading.value = true
        compositeDisposable.add(
            contentRepository.getContentRanking(
                page = 1,
                size = 12,
                sortType = sort,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            _rankContentLiveData.value = it.data.items
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun getPopularContentByCreator(creatorId: Long) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getPopularContentByCreator(
                creatorId = creatorId,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            val data = it.data
                            _salesRankContentListLiveData.value = data.salesRankContentList
                            _salesCountRankContentListLiveData.value =
                                data.salesCountRankContentList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
