package kr.co.vividnext.sodalive.audio_content.player

import android.app.Dialog
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.SeekBar
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.os.BundleCompat
import androidx.media3.common.MediaItem
import androidx.media3.common.MediaMetadata
import androidx.media3.common.Player
import androidx.media3.common.util.UnstableApi
import androidx.media3.session.MediaController
import androidx.media3.session.SessionCommand
import androidx.media3.session.SessionResult
import androidx.media3.session.SessionToken
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import coil.transform.RoundedCornersTransformation
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.playlist.detail.AudioContentPlaylistContent
import kr.co.vividnext.sodalive.audio_content.playlist.detail.AudioContentPlaylistDetailAdapter
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.common.Utils
import kr.co.vividnext.sodalive.databinding.FragmentAudioContentPlayerBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.androidx.viewmodel.ext.android.viewModel

@UnstableApi
class AudioContentPlayerFragment(
    private val screenWidth: Int,
    private val playlist: ArrayList<AudioContentPlaylistContent>
) : BottomSheetDialogFragment() {

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: AudioContentPlaylistDetailAdapter
    private lateinit var binding: FragmentAudioContentPlayerBinding

    private val viewModel: AudioContentPlayerViewModel by viewModel()

    private var mediaController: MediaController? = null
    private val handler = Handler(Looper.getMainLooper())
    private var isUserSeeking = false

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState) as BottomSheetDialog

        dialog.setOnShowListener {
            val bottomSheet = dialog.findViewById<View>(
                com.google.android.material.R.id.design_bottom_sheet
            )
            bottomSheet?.let {
                val layoutParams = it.layoutParams
                layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT
                layoutParams.height = ViewGroup.LayoutParams.MATCH_PARENT
                it.layoutParams = layoutParams

                // BottomSheet를 전체 화면으로 설정
                val behavior = BottomSheetBehavior.from(it)
                behavior.state = BottomSheetBehavior.STATE_EXPANDED
                behavior.skipCollapsed = true
                behavior.isDraggable = false
            }
        }
        return dialog
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentAudioContentPlayerBinding.inflate(
            inflater,
            container,
            false
        )
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setupView()
        bindData()
        connectPlayerService()
    }

    override fun onDestroyView() {
        handler.removeCallbacksAndMessages(null)
        mediaController?.release()
        mediaController = null
        super.onDestroyView()
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)
        binding.ivClose.setOnClickListener { dismiss() }
        binding.ivPlaylist.setOnClickListener {
            viewModel.toggleShowPlayList()
        }

        adapter = AudioContentPlaylistDetailAdapter()

        val recyclerView = binding.rvPlaylistContent
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.VERTICAL,
            false
        )
        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 0
                outRect.right = 0

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }

                    adapter.itemCount - 1 -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 0
                    }

                    else -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })
        recyclerView.adapter = adapter
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            Toast.makeText(requireActivity(), it, Toast.LENGTH_LONG).show()
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(
                    screenWidth,
                    ""
                )
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.isShowPlaylistLiveData.observe(viewLifecycleOwner) {
            if (it) {
                binding.ivCover.visibility = View.GONE
                binding.rvPlaylistContent.visibility = View.VISIBLE
                binding.ivPlaylist.setBackgroundResource(
                    R.drawable.bg_round_corner_6_7_cc333333
                )
            } else {
                binding.ivCover.visibility = View.VISIBLE
                binding.rvPlaylistContent.visibility = View.GONE
                binding.ivPlaylist.setBackgroundResource(0)
            }
        }
    }

    private fun connectPlayerService() {
        context?.let {
            if (!SharedPreferenceManager.isPlayerServiceRunning) {
                startPlayerService(context = it)
            }

            view?.postDelayed({
                connectToMediaSession(it)
            }, 500)
        }
    }

    private fun startPlayerService(context: Context) {
        val serviceIntent = Intent(context, AudioContentPlayerService::class.java)
        context.startService(serviceIntent)
    }

    private fun connectToMediaSession(context: Context) {
        val componentName = ComponentName(context, AudioContentPlayerService::class.java)
        val sessionToken = SessionToken(context, componentName)
        val mediaControllerFuture = MediaController.Builder(context, sessionToken).buildAsync()
        mediaControllerFuture.addListener(
            {
                mediaController = mediaControllerFuture.get()
                setupMediaController()
                updatePlayerUI()
                startUpdatingUI()
            },
            ContextCompat.getMainExecutor(context)
        )
    }

    private fun setupMediaController() {
        if (mediaController == null) {
            Toast.makeText(
                requireContext(),
                "플레이어를 실행하지 못했습니다.\n다시 시도해 주세요.",
                Toast.LENGTH_LONG
            ).show()
            dismiss()
            return
        }

        mediaController!!.addListener(object : Player.Listener {
            override fun onPlaybackStateChanged(playbackState: Int) {
                mediaController?.let {
                    when (playbackState) {
                        Player.STATE_ENDED -> {
                            it.seekTo(0)
                            it.pause()
                        }

                        else -> {}
                    }
                }
            }

            override fun onMediaItemTransition(mediaItem: MediaItem?, reason: Int) {
                updateMediaMetadata(mediaItem?.mediaMetadata)
                updateTimeUI()
            }

            override fun onIsLoadingChanged(isLoading: Boolean) {
                viewModel.setLoading(isLoading)
            }

            override fun onPlayWhenReadyChanged(playWhenReady: Boolean, reason: Int) {
                binding.ivPlayOrPause.setImageResource(
                    if (playWhenReady) {
                        R.drawable.ic_player_pause
                    } else {
                        R.drawable.ic_player_play
                    }
                )
            }
        })

        if (playlist.isNotEmpty()) {
            val extras = Bundle().apply {
                putParcelableArrayList(
                    Constants.EXTRA_AUDIO_CONTENT_PLAYLIST,
                    playlist
                )
            }
            val sessionCommand = SessionCommand("UPDATE_PLAYLIST", Bundle.EMPTY)
            mediaController!!.sendCustomCommand(sessionCommand, extras)
            adapter.updateItems(playlist)
        } else {
            context?.let {
                val sessionCommand = SessionCommand("GET_PLAYLIST", Bundle.EMPTY)
                val resultFuture = mediaController!!.sendCustomCommand(sessionCommand, Bundle.EMPTY)
                resultFuture.addListener(
                    {
                        val result = resultFuture.get()
                        if (result.resultCode == SessionResult.RESULT_SUCCESS) {
                            val data = BundleCompat.getParcelableArrayList(
                                result.extras,
                                Constants.EXTRA_AUDIO_CONTENT_PLAYLIST,
                                AudioContentPlaylistContent::class.java
                            )
                            playlist.clear()
                            playlist.addAll(data ?: listOf())
                            adapter.updateItems(data ?: listOf())
                        }
                    },
                    ContextCompat.getMainExecutor(it)
                )
            }
        }
    }

    private fun updatePlayerUI() {
        binding.ivPlayOrPause.setImageResource(
            if (mediaController!!.isPlaying) {
                R.drawable.ic_player_pause
            } else {
                R.drawable.ic_player_play
            }
        )

        binding.ivPlayOrPause.setOnClickListener {
            if (!SharedPreferenceManager.isPlayerServiceRunning) {
                mediaController = null
                connectPlayerService()
            } else {
                mediaController?.let {
                    if (it.playWhenReady) {
                        it.pause()
                    } else {
                        it.play()
                    }
                }
            }
        }

        binding.ivSkipForward.setOnClickListener {
            mediaController?.let {
                val sessionCommand = SessionCommand(
                    "PLAY_NEXT_CONTENT",
                    Bundle.EMPTY
                )
                it.sendCustomCommand(sessionCommand, Bundle.EMPTY)
            }
        }

        binding.ivSkipBack.setOnClickListener {
            mediaController?.let {
                val sessionCommand = SessionCommand(
                    "PLAY_PREVIOUS_CONTENT",
                    Bundle.EMPTY
                )
                it.sendCustomCommand(sessionCommand, Bundle.EMPTY)
            }
        }

        binding.sbProgress.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(
                seekBar: SeekBar?,
                progress: Int,
                fromUser: Boolean
            ) {
                if (fromUser) {
                    isUserSeeking = true
                }
            }

            override fun onStartTrackingTouch(p0: SeekBar?) {
                isUserSeeking = true
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                isUserSeeking = false
                seekBar?.progress?.let { progress ->
                    mediaController?.seekTo(progress.toLong())
                }
            }

        })

        updateMediaMetadata(mediaController?.currentMediaItem?.mediaMetadata)
        updateTimeUI()
    }

    private fun updateMediaMetadata(metadata: MediaMetadata?) {
        metadata?.let {
            binding.tvTitle.text = it.title
            binding.tvCreatorNickname.text = it.artist

            binding.ivCreatorProfile.load(
                it.extras?.getString(Constants.EXTRA_AUDIO_CONTENT_CREATOR_PROFILE_IMAGE)
            ) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(CircleCropTransformation())
            }

            binding.ivCover.load(it.artworkUri) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(RoundedCornersTransformation(8f.dpToPx()))
            }
        }
    }

    private fun updateTimeUI() {
        mediaController?.let {
            val duration = it.duration
            val currentPosition = it.currentPosition

            binding.sbProgress.max = duration.toInt()
            binding.sbProgress.progress = currentPosition.toInt()

            binding.tvTotalTime.text = Utils.convertDurationToString(duration.toInt())
            binding.tvProgressTime.text = Utils.convertDurationToString(currentPosition.toInt())
        }
    }

    private fun startUpdatingUI() {
        handler.post(object : Runnable {
            override fun run() {
                if (mediaController?.isPlaying == true && !isUserSeeking) {
                    updateTimeUI()
                }

                handler.postDelayed(this, 500)
            }
        })
    }
}
