package kr.co.vividnext.sodalive.audio_content.player

import android.app.PendingIntent
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import androidx.core.os.BundleCompat
import androidx.media3.common.MediaItem
import androidx.media3.common.MediaMetadata
import androidx.media3.common.Player
import androidx.media3.common.util.UnstableApi
import androidx.media3.datasource.DefaultDataSource
import androidx.media3.exoplayer.ExoPlayer
import androidx.media3.exoplayer.source.ProgressiveMediaSource
import androidx.media3.extractor.DefaultExtractorsFactory
import androidx.media3.extractor.ts.AdtsExtractor
import androidx.media3.session.MediaSession
import androidx.media3.session.MediaSessionService
import androidx.media3.session.SessionCommand
import androidx.media3.session.SessionResult
import com.google.common.util.concurrent.Futures
import com.google.common.util.concurrent.ListenableFuture
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.playlist.detail.AudioContentPlaylistContent
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.common.Utils
import kr.co.vividnext.sodalive.main.MainActivity
import org.koin.android.ext.android.inject

@UnstableApi
class AudioContentPlayerService : MediaSessionService() {

    private val repository: AudioContentGenerateUrlRepository by inject()

    private var playlistManager: AudioContentPlaylistManager? = null
    private var mediaSession: MediaSession? = null
    private var player: ExoPlayer? = null

    val compositeDisposable = CompositeDisposable()

    private val playlist = ArrayList<AudioContentPlaylistContent>()

    override fun onCreate() {
        super.onCreate()

        try {
            initPlayer()
            initMediaSession()
        } catch (e: Exception) {
            onStopService()
            return
        }
    }

    override fun onDestroy() {
        mediaSession?.run {
            player.release()
            release()
            mediaSession = null
        }

        compositeDisposable.dispose()
        SharedPreferenceManager.isPlayerServiceRunning = false
        super.onDestroy()
    }

    override fun onTaskRemoved(rootIntent: Intent?) {
        onStopService()
    }

    override fun onGetSession(controllerInfo: MediaSession.ControllerInfo) = mediaSession

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        if (intent?.action == "STOP_SERVICE") {
            onStopService()
        }

        return super.onStartCommand(intent, flags, startId)
    }

    private fun initPlayer() {
        player = ExoPlayer.Builder(this).build()
        player!!.addListener(object : Player.Listener {
            override fun onMediaItemTransition(mediaItem: MediaItem?, reason: Int) {
                super.onMediaItemTransition(mediaItem, reason)
                player?.play()
            }

            override fun onPlaybackStateChanged(playbackState: Int) {
                super.onPlaybackStateChanged(playbackState)
                if (playbackState == Player.STATE_IDLE) {
                    onStopService()
                } else if (playbackState == Player.STATE_ENDED) {
                    if (playlistManager!!.hasNextContent()) {
                        playNextContent()
                    }
                }
            }
        })
    }

    private fun initMediaSession() {
        val contextIntent = Intent(applicationContext, MainActivity::class.java).apply {
            flags = Intent.FLAG_ACTIVITY_CLEAR_TOP or Intent.FLAG_ACTIVITY_SINGLE_TOP
        }
        val pendingIntent = PendingIntent.getActivity(
            applicationContext,
            0,
            contextIntent,
            PendingIntent.FLAG_UPDATE_CURRENT or PendingIntent.FLAG_IMMUTABLE
        )

        mediaSession = MediaSession.Builder(this, player!!)
            .setSessionActivity(pendingIntent)
            .setCallback(object : MediaSession.Callback {
                override fun onConnect(
                    session: MediaSession,
                    controller: MediaSession.ControllerInfo
                ): MediaSession.ConnectionResult {
                    SharedPreferenceManager.isPlayerServiceRunning = true
                    val allowedCommands = MediaSession.ConnectionResult.DEFAULT_SESSION_COMMANDS
                        .buildUpon()
                        .add(SessionCommand("UPDATE_PLAYLIST", Bundle.EMPTY))
                        .add(SessionCommand("PLAY_NEXT_CONTENT", Bundle.EMPTY))
                        .add(SessionCommand("PLAY_PREVIOUS_CONTENT", Bundle.EMPTY))
                        .add(SessionCommand("GET_PLAYLIST", Bundle.EMPTY))
                        .build()

                    return MediaSession.ConnectionResult.AcceptedResultBuilder(session)
                        .setAvailableSessionCommands(allowedCommands)
                        .build()
                }

                override fun onCustomCommand(
                    session: MediaSession,
                    controller: MediaSession.ControllerInfo,
                    customCommand: SessionCommand,
                    args: Bundle
                ): ListenableFuture<SessionResult> {
                    return when (customCommand.customAction) {
                        "UPDATE_PLAYLIST" -> {
                            val playlist = BundleCompat.getParcelableArrayList(
                                args,
                                Constants.EXTRA_AUDIO_CONTENT_PLAYLIST,
                                AudioContentPlaylistContent::class.java
                            )

                            if (playlist != null) {
                                this@AudioContentPlayerService.playlist.addAll(playlist)
                                playlistManager = AudioContentPlaylistManager(playlist)
                                playNextContent()
                            }

                            Futures.immediateFuture(SessionResult(SessionResult.RESULT_SUCCESS))
                        }

                        "PLAY_NEXT_CONTENT" -> {
                            playNextContent()
                            Futures.immediateFuture(SessionResult(SessionResult.RESULT_SUCCESS))
                        }

                        "PLAY_PREVIOUS_CONTENT" -> {
                            playPreviousContent()
                            Futures.immediateFuture(SessionResult(SessionResult.RESULT_SUCCESS))
                        }

                        "GET_PLAYLIST" -> {
                            val extras = Bundle().apply {
                                putParcelableArrayList(
                                    Constants.EXTRA_AUDIO_CONTENT_PLAYLIST,
                                    playlist
                                )
                            }

                            Futures.immediateFuture(
                                SessionResult(
                                    SessionResult.RESULT_SUCCESS,
                                    extras
                                )
                            )
                        }

                        else -> super.onCustomCommand(session, controller, customCommand, args)
                    }
                }
            })
            .build()
    }

    private fun onStopService() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            stopForeground(STOP_FOREGROUND_REMOVE)
        } else {
            stopForeground(true)
        }

        mediaSession?.run {
            player.release()
            release()
            mediaSession = null
        }

        stopSelf()
    }

    private fun playNextContent() {
        val content = playlistManager?.moveToNext()

        if (content != null) {
            generateUrl(
                content.id,
                onSuccess = { urlGenerateSuccess(content, it) },
                onFailure = { playNextContent() }
            )
        }
    }

    private fun urlGenerateSuccess(
        content: AudioContentPlaylistContent,
        contentUrl: String
    ) {
        val extras = Bundle().apply {
            putString(
                Constants.EXTRA_AUDIO_CONTENT_CREATOR_PROFILE_IMAGE,
                content.creatorProfileUrl
            )
        }

        val mediaMetadata = MediaMetadata.Builder()
            .setTitle(content.title)
            .setArtist(content.creatorNickname)
            .setArtworkUri(Uri.parse(content.coverUrl))
            .setExtras(extras)
            .setDurationMs(Utils.convertStringToDuration(content.duration))
            .build()

        val mediaItem = MediaItem.Builder()
            .setUri(Uri.parse(contentUrl))
            .setMediaMetadata(mediaMetadata)
            .build()

        val extractorFactory = DefaultExtractorsFactory().setAdtsExtractorFlags(
            AdtsExtractor.FLAG_ENABLE_CONSTANT_BITRATE_SEEKING
        )
        val mediaSource = ProgressiveMediaSource.Factory(
            DefaultDataSource.Factory(this),
            extractorFactory
        ).createMediaSource(mediaItem)

        player?.setMediaSource(mediaSource)
        player?.prepare()
    }

    private fun playPreviousContent() {
        val content = playlistManager?.moveToPrevious()

        if (content != null) {
            generateUrl(
                content.id,
                onSuccess = { urlGenerateSuccess(content, it) },
                onFailure = { playPreviousContent() }
            )
        }
    }

    private fun generateUrl(contentId: Long, onSuccess: (String) -> Unit, onFailure: () -> Unit) {
        if (contentId <= 0) {
            onFailure()
        }

        compositeDisposable.add(
            repository.generateUrl(
                contentId = contentId,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null && it.data.contentUrl.isNotBlank()) {
                            onSuccess(it.data.contentUrl)
                        } else {
                            onFailure()
                        }
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        onFailure()
                    }
                )
        )
    }
}
