package kr.co.vividnext.sodalive.audio_content.playlist.detail

import android.annotation.SuppressLint
import android.content.ComponentName
import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.widget.ImageView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.media3.common.MediaItem
import androidx.media3.common.MediaMetadata
import androidx.media3.common.Player
import androidx.media3.common.util.UnstableApi
import androidx.media3.session.MediaController
import androidx.media3.session.SessionToken
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.RoundedCornersTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.AudioContentPlayService
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerFragment
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerService
import kr.co.vividnext.sodalive.audio_content.playlist.modify.AudioContentPlaylistModifyActivity
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.base.SodaDialog
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.ActivityAudioContentPlaylistDetailBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject
import kotlin.random.Random

@UnstableApi
class AudioContentPlaylistDetailActivity : BaseActivity<ActivityAudioContentPlaylistDetailBinding>(
    ActivityAudioContentPlaylistDetailBinding::inflate
) {

    private val viewModel: AudioContentPlaylistDetailViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: AudioContentPlaylistDetailAdapter

    private var playlistId: Long = 0

    private val modifyPlaylistResult = registerForActivityResult(
        ActivityResultContracts.StartActivityForResult()
    ) { result ->
        if (result.resultCode == RESULT_OK) {
            viewModel.getPlaylistDetail(playlistId)
        }
    }

    private val contentList = mutableListOf<AudioContentPlaylistContent>()
    private var mediaController: MediaController? = null
    private val handler = Handler(Looper.getMainLooper())

    @SuppressLint("SetTextI18n")
    private val preferenceChangeListener =
        SharedPreferences.OnSharedPreferenceChangeListener { sharedPreferences, key ->
            // 특정 키에 대한 값이 변경될 때 UI 업데이트
            if (key == Constants.PREF_IS_PLAYER_SERVICE_RUNNING) {
                if (sharedPreferences.getBoolean(key, false)) {
                    handler.postDelayed(
                        {
                            initAndVisibleMiniPlayer()
                        },
                        1500
                    )
                } else {
                    deInitMiniPlayer()
                }
            }
        }

    private fun initAndVisibleMiniPlayer() {
        binding.clMiniPlayer.visibility = View.VISIBLE
        binding.clMiniPlayer.setOnClickListener { showPlayerFragment() }
        binding.ivStop.setOnClickListener {
            startService(
                Intent(applicationContext, AudioContentPlayerService::class.java).apply {
                    action = "STOP_SERVICE"
                }
            )
        }
        connectPlayerService()
    }

    private fun connectPlayerService() {
        val componentName = ComponentName(applicationContext, AudioContentPlayerService::class.java)
        val sessionToken = SessionToken(applicationContext, componentName)
        val mediaControllerFuture =
            MediaController.Builder(applicationContext, sessionToken).buildAsync()
        mediaControllerFuture.addListener(
            {
                mediaController = mediaControllerFuture.get()
                setupMediaController()
                updateMediaMetadata(mediaController?.mediaMetadata)

                binding.ivPlayOrPause.setImageResource(
                    if (mediaController!!.isPlaying) {
                        R.drawable.ic_player_pause
                    } else {
                        R.drawable.ic_player_play
                    }
                )

                binding.ivPlayOrPause.setOnClickListener {
                    mediaController?.let {
                        if (it.playWhenReady) {
                            it.pause()
                        } else {
                            it.play()
                        }
                    }
                }
            },
            ContextCompat.getMainExecutor(applicationContext)
        )
    }

    private fun updateMediaMetadata(metadata: MediaMetadata?) {
        metadata?.let {
            binding.tvPlayerTitle.text = it.title
            binding.tvPlayerNickname.text = it.artist

            binding.ivPlayerCover.load(it.artworkUri) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(RoundedCornersTransformation(4f))
            }
        }
    }

    private fun setupMediaController() {
        if (mediaController == null) {
            deInitMiniPlayer()
            return
        }

        mediaController!!.addListener(object : Player.Listener {
            override fun onMediaItemTransition(mediaItem: MediaItem?, reason: Int) {
                updateMediaMetadata(mediaItem?.mediaMetadata)
            }

            override fun onPlayWhenReadyChanged(playWhenReady: Boolean, reason: Int) {
                binding.ivPlayOrPause.setImageResource(
                    if (playWhenReady) {
                        R.drawable.ic_player_pause
                    } else {
                        R.drawable.ic_player_play
                    }
                )
            }
        })
    }

    private fun deInitMiniPlayer() {
        binding.clMiniPlayer.visibility = View.GONE
        mediaController?.release()
        mediaController = null
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        playlistId = intent.getLongExtra(Constants.EXTRA_AUDIO_CONTENT_PLAYLIST_ID, 0)
        super.onCreate(savedInstanceState)

        if (playlistId <= 0) {
            showToast("잘못된 요청입니다.")
            finish()
        }

        bindData()
        viewModel.getPlaylistDetail(playlistId)
        SharedPreferenceManager.registerOnSharedPreferenceChangeListener(preferenceChangeListener)

        if (SharedPreferenceManager.isPlayerServiceRunning) {
            initAndVisibleMiniPlayer()
        } else {
            deInitMiniPlayer()
        }
    }

    override fun onDestroy() {
        deInitMiniPlayer()
        SharedPreferenceManager.unregisterOnSharedPreferenceChangeListener(preferenceChangeListener)
        super.onDestroy()
    }

    private fun showPlayerFragment(
        contentList: ArrayList<AudioContentPlaylistContent> = arrayListOf()
    ) {
        val playerFragment = AudioContentPlayerFragment(screenWidth, contentList)
        playerFragment.show(supportFragmentManager, playerFragment.tag)
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        adapter = AudioContentPlaylistDetailAdapter()

        val recyclerView = binding.rvPlaylistDetail
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(
            applicationContext,
            LinearLayoutManager.VERTICAL,
            false
        )
        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 0
                outRect.right = 0

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }

                    adapter.itemCount - 1 -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 0
                    }

                    else -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })
        recyclerView.adapter = adapter

        binding.llPlay.setOnClickListener {
            startService(
                Intent(applicationContext, AudioContentPlayService::class.java).apply {
                    action = AudioContentPlayService.MusicAction.STOP.name
                }
            )

            showPlayerFragment(contentList = ArrayList(contentList))
        }
        binding.llShuffle.setOnClickListener {
            startService(
                Intent(applicationContext, AudioContentPlayService::class.java).apply {
                    action = AudioContentPlayService.MusicAction.STOP.name
                }
            )

            val shuffledList = ArrayList(contentList).apply { shuffle(Random) }
            showPlayerFragment(contentList = shuffledList)
        }

        binding.tvBack.setOnClickListener { finish() }
        binding.ivEdit.setOnClickListener {
            modifyPlaylistResult.launch(
                Intent(
                    applicationContext,
                    AudioContentPlaylistModifyActivity::class.java
                ).apply {
                    putExtra(
                        Constants.EXTRA_AUDIO_CONTENT_PLAYLIST_ID,
                        playlistId
                    )
                }
            )
        }
        binding.ivMore.setOnClickListener {
            val notifyFragment = AudioContentPlaylistDetailNotifyFragment {
                SodaDialog(
                    activity = this@AudioContentPlaylistDetailActivity,
                    layoutInflater = layoutInflater,
                    title = "재생 목록 삭제",
                    desc = "'${binding.tvTitle.text}'을 삭제하시겠습니까?",
                    confirmButtonTitle = "삭제",
                    confirmButtonClick = {
                        viewModel.deletePlaylist(playlistId = playlistId) {
                            setResult(RESULT_OK)
                            finish()
                        }
                    },
                    cancelButtonTitle = "취소",
                    cancelButtonClick = {}
                ).show(screenWidth)
            }

            if (notifyFragment.isAdded) return@setOnClickListener
            notifyFragment.show(supportFragmentManager, notifyFragment.tag)
        }
    }

    @SuppressLint("SetTextI18n")
    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.detailResponseLiveData.observe(this) {
            binding.tvDesc.text = it.desc
            binding.tvTitle.text = it.title
            binding.tvContentCount.text = " ${it.contentCount}개"
            binding.tvCreateDate.text = "만든 날짜 ${it.createdDate} "
            adapter.updateItems(it.contentList)
            this.contentList.clear()
            this.contentList.addAll(it.contentList)
            updateCoverImageLayout(imageUrlList = it.playlistCoverImageList)
        }
    }

    private fun updateCoverImageLayout(imageUrlList: List<String>) {
        val imageViews = listOf(
            binding.ivCover1,
            binding.ivCover2,
            binding.ivCover3,
            binding.ivCover4,
            binding.ivCover5,
            binding.ivCover6
        )

        imageViews.forEach { it.visibility = View.GONE }

        when (imageUrlList.size) {
            1, 2 -> {
                setImage(imageViews[0], imageUrlList[0])
            }

            3 -> {
                setImage(imageViews[1], imageUrlList[0])
                setImage(imageViews[2], imageUrlList[1])
                setImage(imageViews[3], imageUrlList[2])
            }

            4 -> {
                setImage(imageViews[1], imageUrlList[0])
                setImage(imageViews[2], imageUrlList[1])
                setImage(imageViews[4], imageUrlList[2])
                setImage(imageViews[5], imageUrlList[3])
            }
        }
    }

    private fun setImage(imageView: ImageView, imageUrl: String) {
        imageView.apply {
            visibility = View.VISIBLE
            this.load(imageUrl)
        }
    }
}
