package kr.co.vividnext.sodalive.audition.applicant

import android.content.Context
import android.content.Intent
import android.media.AudioAttributes
import android.media.MediaPlayer
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.media3.common.util.UnstableApi
import kr.co.vividnext.sodalive.audio_content.AudioContentPlayService
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerService
import java.io.IOException

@OptIn(UnstableApi::class)
class AuditionApplicantMediaPlayerManager(
    private val context: Context,
    private val updatePlayPauseState: (Int, Boolean) -> Unit,
    private val updateTotalDuration: (Int) -> Unit,
    private val updateCurrentTime: (Int) -> Unit,
    private val showLoadingDialog: (Boolean) -> Unit
) {
    private var mediaPlayer: MediaPlayer? = null
    private var currentPlayingApplicantId: Long = -1
    private var currentPlayingPosition: Int = -1

    private val handler = Handler(Looper.getMainLooper())
    private var changeMediaPlayerPositionRunnable = object : Runnable {
        override fun run() {
            if (mediaPlayer != null) {
                updateCurrentTime(mediaPlayer!!.currentPosition)
            }
            handler.postDelayed(this, 1000)
        }
    }

    fun pauseContent() {
        mediaPlayer?.pause()
        updatePlayPauseState(currentPlayingPosition, false)
    }

    private fun resumeContent() {
        pauseAudioContentService()
        mediaPlayer?.start()
        updatePlayPauseState(currentPlayingPosition, true)
    }

    fun stopContent() {
        mediaPlayer?.let {
            it.stop()
            it.release()
            mediaPlayer = null
        }
        currentPlayingApplicantId = -1
        currentPlayingPosition = -1
        updatePlayPauseState(currentPlayingPosition, false)
        handler.removeCallbacks(changeMediaPlayerPositionRunnable)
    }

    fun toggleContent(position: Int, applicantId: Long, voiceUrl: String) {
        if (currentPlayingApplicantId == applicantId && currentPlayingPosition == position) {
            if (mediaPlayer?.isPlaying == true) {
                pauseContent()
                handler.removeCallbacks(changeMediaPlayerPositionRunnable)
            } else {
                resumeContent()
                handler.postDelayed(changeMediaPlayerPositionRunnable, 1000)
            }
        } else {
            playContent(position, applicantId, voiceUrl)
            handler.postDelayed(changeMediaPlayerPositionRunnable, 1000)
        }
    }

    private fun playContent(position: Int, applicantId: Long, voiceUrl: String) {
        showLoadingDialog(true)
        pauseAudioContentService()
        stopContent()

        currentPlayingPosition = position
        currentPlayingApplicantId = applicantId

        mediaPlayer = MediaPlayer().apply {
            setAudioAttributes(
                AudioAttributes.Builder()
                    .setContentType(AudioAttributes.CONTENT_TYPE_MUSIC)
                    .setUsage(AudioAttributes.USAGE_MEDIA)
                    .build()
            )

            try {
                setDataSource(context, Uri.parse(voiceUrl))
                prepareAsync() // 비동기적으로 준비
                setOnPreparedListener {
                    start()
                    updateTotalDuration(duration)
                    updatePlayPauseState(currentPlayingPosition, true)
                    showLoadingDialog(false)
                }
            } catch (e: IOException) {
                e.printStackTrace()
                Toast.makeText(context, "콘텐츠를 재생하지 못했습니다.\n다시 시도해 주세요", Toast.LENGTH_SHORT).show()
                showLoadingDialog(false)
            }

            setOnCompletionListener {
                updatePlayPauseState(currentPlayingPosition, false)
                currentPlayingApplicantId = -1
            }
        }
    }

    private fun pauseAudioContentService() {
        context.startService(
            Intent(
                context,
                AudioContentPlayService::class.java
            ).apply {
                action = AudioContentPlayService.MusicAction.PAUSE.name
            }
        )

        context.startService(
            Intent(context, AudioContentPlayerService::class.java).apply {
                action = "STOP_SERVICE"
            }
        )
    }
}
