package kr.co.vividnext.sodalive.main

import android.Manifest
import android.annotation.SuppressLint
import android.app.Service
import android.content.BroadcastReceiver
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.SharedPreferences
import android.content.res.ColorStateList
import android.graphics.Rect
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.inputmethod.InputMethodManager
import androidx.activity.addCallback
import androidx.core.content.ContextCompat
import androidx.core.content.res.ResourcesCompat
import androidx.media3.common.MediaItem
import androidx.media3.common.MediaMetadata
import androidx.media3.common.Player
import androidx.media3.common.util.UnstableApi
import androidx.media3.session.MediaController
import androidx.media3.session.SessionToken
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.RoundedCornersTransformation
import com.google.firebase.messaging.FirebaseMessaging
import com.gun0912.tedpermission.PermissionListener
import com.gun0912.tedpermission.normal.TedPermission
import com.jakewharton.rxbinding4.widget.textChanges
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.AudioContentPlayService
import kr.co.vividnext.sodalive.audio_content.detail.AudioContentDetailActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.home.AudioContentMainTabHomeFragment
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerFragment
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerService
import kr.co.vividnext.sodalive.audition.AuditionFragment
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.ActivityMainBinding
import kr.co.vividnext.sodalive.databinding.ItemMainTabBinding
import kr.co.vividnext.sodalive.explorer.ExplorerViewModel
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.LiveFragment
import kr.co.vividnext.sodalive.message.MessageFragment
import kr.co.vividnext.sodalive.message.SelectMessageRecipientAdapter
import kr.co.vividnext.sodalive.mypage.MyPageFragment
import kr.co.vividnext.sodalive.settings.event.EventDetailActivity
import kr.co.vividnext.sodalive.settings.notification.NotificationSettingsDialog
import org.koin.android.ext.android.inject
import java.util.concurrent.TimeUnit

@UnstableApi
class MainActivity : BaseActivity<ActivityMainBinding>(ActivityMainBinding::inflate) {

    private val viewModel: MainViewModel by inject()
    private val explorerViewModel: ExplorerViewModel by inject()

    private lateinit var searchChannelAdapter: SelectMessageRecipientAdapter
    private lateinit var imm: InputMethodManager

    private lateinit var liveFragment: LiveFragment
    private lateinit var loadingDialog: LoadingDialog
    private lateinit var notificationSettingsDialog: NotificationSettingsDialog

    private var mediaController: MediaController? = null
    private val handler = Handler(Looper.getMainLooper())
    private val audioContentReceiver = AudioContentReceiver()
    private var isShowSearchBar = false

    private val preferenceChangeListener =
        SharedPreferences.OnSharedPreferenceChangeListener { sharedPreferences, key ->
            // 특정 키에 대한 값이 변경될 때 UI 업데이트
            if (key == Constants.PREF_IS_PLAYER_SERVICE_RUNNING) {
                if (sharedPreferences.getBoolean(key, false)) {
                    handler.postDelayed(
                        {
                            initAndVisibleMiniPlayer()
                        },
                        1500
                    )
                } else {
                    deInitMiniPlayer()
                }
            }
        }

    private fun initAndVisibleMiniPlayer() {
        binding.clMiniPlayer.visibility = View.VISIBLE
        binding.clMiniPlayer.setOnClickListener { showPlayerFragment() }
        binding.ivPlayerStop.setOnClickListener {
            startService(
                Intent(applicationContext, AudioContentPlayerService::class.java).apply {
                    action = "STOP_SERVICE"
                }
            )
        }
        connectPlayerService()
    }

    private fun connectPlayerService() {
        val componentName = ComponentName(applicationContext, AudioContentPlayerService::class.java)
        val sessionToken = SessionToken(applicationContext, componentName)
        val mediaControllerFuture =
            MediaController.Builder(applicationContext, sessionToken).buildAsync()
        mediaControllerFuture.addListener(
            {
                mediaController = mediaControllerFuture.get()
                setupMediaController()
                updateMediaMetadata(mediaController?.mediaMetadata)

                binding.ivPlayerPlayOrPause.setImageResource(
                    if (mediaController!!.isPlaying) {
                        R.drawable.ic_player_pause
                    } else {
                        R.drawable.ic_player_play
                    }
                )

                binding.ivPlayerPlayOrPause.setOnClickListener {
                    mediaController?.let {
                        if (it.playWhenReady) {
                            it.pause()
                        } else {
                            it.play()
                        }
                    }
                }
            },
            ContextCompat.getMainExecutor(applicationContext)
        )
    }

    private fun updateMediaMetadata(metadata: MediaMetadata?) {
        metadata?.let {
            binding.tvPlayerTitle.text = it.title
            binding.tvPlayerNickname.text = it.artist

            binding.ivPlayerCover.load(it.artworkUri) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(RoundedCornersTransformation(4f))
            }
        }
    }

    private fun setupMediaController() {
        if (mediaController == null) {
            deInitMiniPlayer()
            return
        }

        mediaController!!.addListener(object : Player.Listener {
            override fun onMediaItemTransition(mediaItem: MediaItem?, reason: Int) {
                updateMediaMetadata(mediaItem?.mediaMetadata)
            }

            override fun onPlayWhenReadyChanged(playWhenReady: Boolean, reason: Int) {
                binding.ivPlayerPlayOrPause.setImageResource(
                    if (playWhenReady) {
                        R.drawable.ic_player_pause
                    } else {
                        R.drawable.ic_player_play
                    }
                )
            }
        })
    }

    private fun deInitMiniPlayer() {
        binding.clMiniPlayer.visibility = View.GONE
        mediaController?.release()
        mediaController = null
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        executeDeeplink(intent)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        imm = getSystemService(
            Service.INPUT_METHOD_SERVICE
        ) as InputMethodManager

        checkPermissions()
        pushTokenUpdate()
        gaidUpdate()

        getMemberInfo()
        getEventPopup()

        SharedPreferenceManager.registerOnSharedPreferenceChangeListener(preferenceChangeListener)
        if (SharedPreferenceManager.isPlayerServiceRunning) {
            initAndVisibleMiniPlayer()
        } else {
            deInitMiniPlayer()
        }

        handler.postDelayed({ executeDeeplink(intent) }, 500)

        onBackPressedDispatcher.addCallback {
            if (isShowSearchBar) {
                hideSearchBar()
            } else {
                isEnabled = false
                onBackPressedDispatcher.onBackPressed()
            }
        }
    }

    override fun onDestroy() {
        deInitMiniPlayer()
        SharedPreferenceManager.unregisterOnSharedPreferenceChangeListener(preferenceChangeListener)
        super.onDestroy()
    }

    private fun showPlayerFragment() {
        val playerFragment = AudioContentPlayerFragment(screenWidth, arrayListOf())
        playerFragment.show(supportFragmentManager, playerFragment.tag)
    }

    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    override fun onResume() {
        super.onResume()
        val intentFilter = IntentFilter(Constants.ACTION_MAIN_AUDIO_CONTENT_RECEIVER)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            registerReceiver(audioContentReceiver, intentFilter, Context.RECEIVER_NOT_EXPORTED)
        } else {
            registerReceiver(audioContentReceiver, intentFilter)
        }

        startService(
            Intent(this, AudioContentPlayService::class.java).apply {
                action = AudioContentPlayService.MusicAction.INIT.name
            }
        )
    }

    override fun onPause() {
        unregisterReceiver(audioContentReceiver)
        super.onPause()
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        liveFragment = LiveFragment()

        notificationSettingsDialog = NotificationSettingsDialog(
            this,
            layoutInflater
        ) { isNotifiedLive, isNotifiedUploadContent, isNotifiedMessage ->
            viewModel.updateNotificationSettings(
                isNotifiedLive,
                isNotifiedUploadContent,
                isNotifiedMessage
            )
        }

        setupBottomTabLayout()
        setupSearchChannelView()
    }

    private fun executeDeeplink(intent: Intent) {
        val bundle = intent.getBundleExtra(Constants.EXTRA_DATA)
        if (bundle != null) {
            try {
                val roomId = bundle.getLong(Constants.EXTRA_ROOM_ID)
                val channelId = bundle.getLong(Constants.EXTRA_USER_ID)
                val messageId = bundle.getLong(Constants.EXTRA_MESSAGE_ID)
                val auditionId = bundle.getLong(Constants.EXTRA_AUDITION_ID)
                val contentId = bundle.getLong(Constants.EXTRA_AUDIO_CONTENT_ID)
                val isLiveReservation = bundle.getBoolean(Constants.EXTRA_LIVE_RESERVATION_RESPONSE)

                if (roomId > 0) {
                    viewModel.clickTab(MainViewModel.CurrentTab.LIVE)

                    handler.postDelayed({
                        if (isLiveReservation) {
                            liveFragment.reservationRoom(roomId)
                        } else {
                            liveFragment.enterLiveRoom(roomId)
                        }
                    }, 500)
                } else if (channelId > 0) {
                    val nextIntent = Intent(applicationContext, UserProfileActivity::class.java)
                    nextIntent.putExtra(Constants.EXTRA_USER_ID, channelId)
                    startActivity(nextIntent)
                } else if (contentId > 0) {
                    val nextIntent = Intent(
                        applicationContext,
                        AudioContentDetailActivity::class.java
                    )
                    nextIntent.putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, contentId)
                    startActivity(nextIntent)
                } else if (messageId > 0) {
                    viewModel.clickTab(MainViewModel.CurrentTab.MESSAGE)
                } else if (auditionId > 0) {
                    viewModel.clickTab(MainViewModel.CurrentTab.AUDITION)
                }
            } catch (_: IllegalStateException) {
            }
        }
    }

    private fun setupBottomTabLayout() {
        setupTab(
            binding = binding.tabContent,
            title = "콘텐츠",
            imageSrc = R.drawable.ic_tabbar_content,
            colorStateList = ContextCompat.getColorStateList(
                applicationContext,
                R.color.color_tabbar_title
            ),
            tab = MainViewModel.CurrentTab.CONTENT
        )

        setupTab(
            binding = binding.tabLive,
            title = "라이브",
            imageSrc = R.drawable.ic_tabbar_live,
            colorStateList = ContextCompat.getColorStateList(
                applicationContext,
                R.color.color_tabbar_title
            ),
            tab = MainViewModel.CurrentTab.LIVE
        )

        setupTab(
            binding = binding.tabAudition,
            title = "오디션",
            imageSrc = R.drawable.ic_tabbar_audition,
            colorStateList = ContextCompat.getColorStateList(
                applicationContext,
                R.color.color_tabbar_title
            ),
            tab = MainViewModel.CurrentTab.AUDITION
        )

        setupTab(
            binding = binding.tabMessage,
            title = "메시지",
            imageSrc = R.drawable.ic_tabbar_message,
            colorStateList = ContextCompat.getColorStateList(
                applicationContext,
                R.color.color_tabbar_title
            ),
            tab = MainViewModel.CurrentTab.MESSAGE
        )

        setupTab(
            binding = binding.tabMy,
            title = "마이",
            imageSrc = R.drawable.ic_tabbar_my,
            colorStateList = ContextCompat.getColorStateList(
                applicationContext,
                R.color.color_tabbar_title
            ),
            tab = MainViewModel.CurrentTab.MY
        )

        viewModel.currentTab.observe(this) {
            setTabSelected(binding.tabContent, isSelected = false)
            setTabSelected(binding.tabLive, isSelected = false)
            setTabSelected(binding.tabAudition, isSelected = false)
            setTabSelected(binding.tabMessage, isSelected = false)
            setTabSelected(binding.tabMy, isSelected = false)

            changeFragment(it)
            when (it) {
                MainViewModel.CurrentTab.CONTENT -> {
                    setTabSelected(binding.tabContent, isSelected = true)
                }

                MainViewModel.CurrentTab.LIVE -> {
                    setTabSelected(binding.tabLive, isSelected = true)
                }

                MainViewModel.CurrentTab.AUDITION -> {
                    setTabSelected(binding.tabAudition, isSelected = true)
                }

                MainViewModel.CurrentTab.MESSAGE -> {
                    setTabSelected(binding.tabMessage, isSelected = true)
                }

                MainViewModel.CurrentTab.MY -> {
                    setTabSelected(binding.tabMy, isSelected = true)
                }

                else -> {
                }
            }
        }
    }

    private fun setupTab(
        binding: ItemMainTabBinding,
        title: String,
        imageSrc: Int,
        colorStateList: ColorStateList?,
        tab: MainViewModel.CurrentTab
    ) {
        binding.tvTab.text = title
        binding.tvTab.setTextColor(colorStateList)
        binding.ivTab.setImageResource(imageSrc)
        binding.root.setOnClickListener { viewModel.clickTab(tab) }
    }

    private fun setTabSelected(binding: ItemMainTabBinding, isSelected: Boolean) {
        binding.tvTab.isSelected = isSelected
        binding.ivTab.isSelected = isSelected

        val fontRes = if (isSelected) {
            R.font.gmarket_sans_bold
        } else {
            R.font.gmarket_sans_light
        }
        binding.tvTab.typeface = ResourcesCompat.getFont(applicationContext, fontRes)
    }

    private fun changeFragment(currentTab: MainViewModel.CurrentTab) {
        val tag = currentTab.toString()

        val fragmentManager = supportFragmentManager
        val fragmentTransaction = fragmentManager.beginTransaction()

        val currentFragment = fragmentManager.primaryNavigationFragment
        if (currentFragment != null) {
            fragmentTransaction.hide(currentFragment)
        }

        var fragment = fragmentManager.findFragmentByTag(tag)
        if (fragment == null) {
            fragment = when (currentTab) {
                MainViewModel.CurrentTab.LIVE -> liveFragment
                MainViewModel.CurrentTab.CONTENT -> AudioContentMainTabHomeFragment()
                MainViewModel.CurrentTab.AUDITION -> AuditionFragment()
                MainViewModel.CurrentTab.MESSAGE -> MessageFragment()
                MainViewModel.CurrentTab.MY -> MyPageFragment()
            }

            fragmentTransaction.add(R.id.fl_container, fragment, tag)
        } else {
            fragmentTransaction.show(fragment)
        }

        fragmentTransaction.setPrimaryNavigationFragment(fragment)
        fragmentTransaction.setReorderingAllowed(true)
        fragmentTransaction.commitNow()
    }

    private fun checkPermissions() {
        val permissions = mutableListOf(Manifest.permission.RECORD_AUDIO)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            permissions.add(Manifest.permission.POST_NOTIFICATIONS)
        }

        TedPermission.create()
            .setPermissionListener(object : PermissionListener {
                override fun onPermissionGranted() {
                }

                override fun onPermissionDenied(deniedPermissions: MutableList<String>?) {
                }
            })
            .setDeniedMessage(R.string.record_audio_permission_denied_message)
            .setPermissions(*permissions.toTypedArray())
            .check()
    }

    private fun pushTokenUpdate() {
        FirebaseMessaging.getInstance().token.addOnCompleteListener {
            if (!it.isSuccessful) {
                Logger.v("Fetching FCM registration token failed", it.exception)
                return@addOnCompleteListener
            }

            val pushToken = it.result
            if (pushToken != null) {
                SharedPreferenceManager.pushToken = pushToken
                viewModel.pushTokenUpdate(pushToken)
            }
        }
    }

    private fun gaidUpdate() {
        viewModel.fetchAndUpdateGaid(context = applicationContext)
    }

    private fun getMemberInfo() {
        viewModel.getMemberInfo {
            notificationSettingsDialog.show(screenWidth)
        }
    }

    private fun getEventPopup() {
        viewModel.getEventPopup {
            if (SharedPreferenceManager.notShowingEventPopupId != it.id) {
                val eventPopupDialog = EventPopupDialogFragment(
                    screenWidth = screenWidth,
                    eventItem = it
                ) {
                    startActivity(
                        Intent(
                            applicationContext,
                            EventDetailActivity::class.java
                        ).apply {
                            putExtra(Constants.EXTRA_EVENT, it)
                        }
                    )
                }

                eventPopupDialog.show(supportFragmentManager, eventPopupDialog.tag)
            }
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupSearchChannelView() {
        searchChannelAdapter = SelectMessageRecipientAdapter {
            hideKeyboard()
            val intent = Intent(applicationContext, UserProfileActivity::class.java)
            intent.putExtra(Constants.EXTRA_USER_ID, it.id)
            startActivity(intent)
        }

        binding.rvSearchChannel.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvSearchChannel.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvSearchChannel.adapter = searchChannelAdapter

        compositeDisposable.add(
            binding.etSearchChannel.textChanges().skip(1)
                .debounce(500, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribeOn(Schedulers.io())
                .subscribe {
                    if (it.length >= 2) {
                        explorerViewModel.searchChannel(it.toString())
                        binding.rvSearchChannel.visibility = View.VISIBLE
                        binding.tvResultX.visibility = View.GONE
                    } else {
                        binding.rvSearchChannel.visibility = View.GONE
                        binding.tvResultX.visibility = View.VISIBLE
                    }
                }
        )

        explorerViewModel.searchChannelLiveData.observe(this) {
            searchChannelAdapter.items.clear()
            if (it.isNotEmpty()) {
                searchChannelAdapter.items.addAll(it)
                binding.rvSearchChannel.visibility = View.VISIBLE
                binding.tvResultX.visibility = View.GONE
            } else {
                binding.rvSearchChannel.visibility = View.GONE
                binding.tvResultX.visibility = View.VISIBLE
            }
            searchChannelAdapter.notifyDataSetChanged()
        }
    }

    private fun hideKeyboard() {
        handler.postDelayed({
            imm.hideSoftInputFromWindow(
                window.decorView.applicationWindowToken,
                InputMethodManager.HIDE_NOT_ALWAYS
            )
        }, 100)
    }

    private fun hideSearchBar() {
        isShowSearchBar = false
        binding.nsSearch.visibility = View.GONE
        binding.etSearchChannel.setText("")
    }

    fun showSearchBar() {
        isShowSearchBar = true
        binding.nsSearch.visibility = View.VISIBLE
        binding.etSearchChannel.setText("")

        binding.etSearchChannel.requestFocus()
        binding.etSearchChannel.post {
            val imm = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
            imm.showSoftInput(binding.etSearchChannel, InputMethodManager.SHOW_IMPLICIT)
        }
    }

    inner class AudioContentReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val contentId = intent?.getLongExtra(Constants.EXTRA_AUDIO_CONTENT_ID, 0)
            val title = intent?.getStringExtra(Constants.EXTRA_AUDIO_CONTENT_TITLE)
            val nickname = intent?.getStringExtra(Constants.EXTRA_NICKNAME)
            val coverImageUrl = intent?.getStringExtra(
                Constants.EXTRA_AUDIO_CONTENT_COVER_IMAGE_URL
            )

            val isPlaying = intent?.getBooleanExtra(Constants.EXTRA_AUDIO_CONTENT_PLAYING, false)
            val isShowing = intent?.getBooleanExtra(Constants.EXTRA_AUDIO_CONTENT_SHOWING, false)

            if (isShowing == true) {
                binding.rlMiniPlayer.visibility = View.VISIBLE
                if (contentId != null && contentId > 0) {
                    binding.rlMiniPlayer.setOnClickListener {
                        startActivity(
                            Intent(applicationContext, AudioContentDetailActivity::class.java)
                                .apply {
                                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, contentId)
                                }
                        )
                    }
                }

                if (isPlaying == true) {
                    binding.ivPlayOrPause.setImageResource(R.drawable.ic_noti_pause)
                    binding.ivPlayOrPause.setOnClickListener {
                        startService(
                            Intent(
                                this@MainActivity,
                                AudioContentPlayService::class.java
                            ).apply {
                                action = AudioContentPlayService.MusicAction.PAUSE.name
                            }
                        )
                    }
                } else {
                    binding.ivPlayOrPause.setImageResource(R.drawable.ic_noti_play)
                    binding.ivPlayOrPause.setOnClickListener {
                        startService(
                            Intent(
                                this@MainActivity,
                                AudioContentPlayService::class.java
                            ).apply {
                                action = AudioContentPlayService.MusicAction.PLAY.name
                            }
                        )
                    }
                }

                binding.ivStop.setOnClickListener {
                    startService(
                        Intent(
                            this@MainActivity,
                            AudioContentPlayService::class.java
                        ).apply {
                            action = AudioContentPlayService.MusicAction.STOP.name
                        }
                    )
                }

                if (!title.isNullOrBlank()) {
                    binding.tvTitle.text = title
                }

                if (!nickname.isNullOrBlank()) {
                    binding.tvNickname.text = nickname
                }

                if (!coverImageUrl.isNullOrBlank()) {
                    binding.ivCover.load(coverImageUrl) {
                        crossfade(true)
                        placeholder(R.drawable.bg_placeholder)
                        transformations(RoundedCornersTransformation(5.3f.dpToPx()))
                    }
                }
            } else {
                handler.post {
                    binding.ivPlayOrPause.setImageResource(0)
                    binding.ivCover.setImageResource(0)
                    binding.tvTitle.text = ""
                    binding.tvNickname.text = ""
                    binding.rlMiniPlayer.visibility = View.GONE
                    binding.ivPlayOrPause.setOnClickListener {}
                    viewModel.addAllPlaybackTracking()
                }
            }
        }
    }
}
